@@ -14,20 +14,21 @@
 #  limitations under the License.
 #
 
+use strict;
+use warnings;
 package MongoDB::Collection;
 
 
 # ABSTRACT: A MongoDB Collection
 
 use version;
-our $VERSION = 'v1.2.2';
+our $VERSION = 'v1.8.1';
 
 use MongoDB::Error;
 use MongoDB::IndexView;
 use MongoDB::InsertManyResult;
 use MongoDB::QueryResult;
 use MongoDB::WriteConcern;
-use MongoDB::_Query;
 use MongoDB::Op::_Aggregate;
 use MongoDB::Op::_BatchInsert;
 use MongoDB::Op::_BulkWrite;
@@ -35,11 +36,14 @@ use MongoDB::Op::_Count;
 use MongoDB::Op::_CreateIndexes;
 use MongoDB::Op::_Delete;
 use MongoDB::Op::_Distinct;
+use MongoDB::Op::_DropCollection;
 use MongoDB::Op::_FindAndDelete;
 use MongoDB::Op::_FindAndUpdate;
 use MongoDB::Op::_InsertOne;
 use MongoDB::Op::_ListIndexes;
 use MongoDB::Op::_ParallelScan;
+use MongoDB::Op::_RenameCollection;
+use MongoDB::Op::_Query;
 use MongoDB::Op::_Update;
 use MongoDB::_Types qw(
     BSONCodec
@@ -269,6 +273,10 @@ sub _build__op_args {
     };
 }
 
+with $_ for qw(
+  MongoDB::Role::_DeprecationWarner
+);
+
 #--------------------------------------------------------------------------#
 # public methods
 #--------------------------------------------------------------------------#
@@ -407,41 +415,59 @@ sub insert_many {
     MongoDB::UsageError->throw("documents argument must be an array reference")
       unless ref( $_[1] ) eq 'ARRAY';
 
-    return MongoDB::InsertManyResult->_new(
-        acknowledged => $_[0]->write_concern->is_acknowledged,
-        inserted     => $_[0]->client->send_write_op(
-            MongoDB::Op::_BulkWrite->_new(
-                # default
-                ordered => 1,
-                # user overrides
-                ( defined $_[2] ? ( %{ $_[2] } ) : () ),
-                # un-overridable
-                queue => [ map { [ insert => $_ ] } @{ $_[1] } ],
-                %{ $_[0]->_op_args },
-            )
-          )->inserted,
+    my $res = $_[0]->client->send_write_op(
+        MongoDB::Op::_BulkWrite->_new(
+            # default
+            ordered => 1,
+            # user overrides
+            ( defined $_[2] ? ( %{ $_[2] } ) : () ),
+            # un-overridable
+            queue => [ map { [ insert => $_ ] } @{ $_[1] } ],
+            %{ $_[0]->_op_args },
+        )
+    );
+
+    return $_[0]->write_concern->is_acknowledged
+      ? MongoDB::InsertManyResult->_new(
+        acknowledged         => 1,
+        inserted             => $res->inserted,
         write_errors         => [],
         write_concern_errors => [],
-    );
+      )
+      : MongoDB::UnacknowledgedResult->_new(
+        write_errors         => [],
+        write_concern_errors => [],
+      );
 }
 
 #pod =method delete_one
 #pod
 #pod     $res = $coll->delete_one( $filter );
 #pod     $res = $coll->delete_one( { _id => $id } );
+#pod     $res = $coll->delete_one( $filter, { collation => { locale => "en_US" } } );
 #pod
 #pod Deletes a single document that matches a L<filter expression|/Filter expression> and returns a
 #pod L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult> object.
 #pod
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
+#pod
 #pod =cut
 
-# args not unpacked for efficiency; args are self, filter
+# args not unpacked for efficiency; args are self, filter, options
 sub delete_one {
     MongoDB::UsageError->throw("filter argument must be a reference")
       unless ref( $_[1] );
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_Delete->_new(
+            ( defined $_[2] ? (%{$_[2]}) : () ),
             filter   => $_[1],
             just_one => 1,
             %{ $_[0]->_op_args },
@@ -453,20 +479,29 @@ sub delete_one {
 #pod
 #pod     $res = $coll->delete_many( $filter );
 #pod     $res = $coll->delete_many( { name => "Larry" } );
+#pod     $res = $coll->delete_many( $filter, { collation => { locale => "en_US" } } );
 #pod
 #pod Deletes all documents that match a L<filter expression|/Filter expression>
 #pod and returns a L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult>
 #pod object.
 #pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
+#pod
 #pod =cut
 
-# args not unpacked for efficiency; args are self, filter
+# args not unpacked for efficiency; args are self, filter, options
 sub delete_many {
     MongoDB::UsageError->throw("filter argument must be a reference")
       unless ref( $_[1] );
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_Delete->_new(
+            ( defined $_[2] ? (%{$_[2]}) : () ),
             filter   => $_[1],
             just_one => 0,
             %{ $_[0]->_op_args },
@@ -493,6 +528,9 @@ sub delete_many {
 #pod =for :list
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
 #pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<upsert> – defaults to false; if true, a new document will be added if one
 #pod   is not found
 #pod
@@ -534,6 +572,9 @@ sub replace_one {
 #pod =for :list
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
 #pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<upsert> – defaults to false; if true, a new document will be added if
 #pod   one is not found by taking the filter expression and applying the update
 #pod   document operations to it prior to insertion.
@@ -576,6 +617,9 @@ sub update_one {
 #pod =for :list
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
 #pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<upsert> – defaults to false; if true, a new document will be added if
 #pod   one is not found by taking the filter expression and applying the update
 #pod   document operations to it prior to insertion.
@@ -618,6 +662,9 @@ sub update_many {
 #pod * C<allowPartialResults> - get partial results from a mongos if some shards are
 #pod   down (instead of throwing an error).
 #pod * C<batchSize> – the number of documents to return per batch.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<comment> – attaches a comment to the query. If C<$comment> also exists in
 #pod   the C<modifiers> document, the comment field overwrites C<$comment>.
 #pod * C<cursorType> – indicates the type of cursor to use. It must be one of three
@@ -632,7 +679,7 @@ sub update_many {
 #pod   C<$maxTimeMS> also exists in the modifiers document, the C<maxTimeMS> field
 #pod   overwrites C<$maxTimeMS>. (Note, this will be ignored for servers before
 #pod   version 2.6.)
-#pod * C<modifiers> – a hash reference of L<query
+#pod * C<modifiers> – a hash reference of dollar-prefixed L<query
 #pod   modifiers|http://docs.mongodb.org/manual/reference/operator/query-modifier/>
 #pod   modifying the output or behavior of a query.
 #pod * C<noCursorTimeout> – if true, prevents the server from timing out a cursor
@@ -681,7 +728,8 @@ sub find {
     __ixhash( $options, 'sort' );
 
     return MongoDB::Cursor->new(
-        query => MongoDB::_Query->_new(
+        client => $self->{_client},
+        query => MongoDB::Op::_Query->_new(
             modifiers           => {},
             allowPartialResults => 0,
             batchSize           => 0,
@@ -725,6 +773,9 @@ sub find {
 #pod include:
 #pod
 #pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
 #pod * C<sort> – an L<ordered document|/Ordered document> defining the order in which
@@ -749,25 +800,27 @@ sub find_one {
     # coerce to IxHash
     __ixhash( $options, 'sort' );
 
-    return MongoDB::_Query->_new(
-        modifiers           => {},
-        allowPartialResults => 0,
-        batchSize           => 0,
-        comment             => '',
-        cursorType          => 'non_tailable',
-        limit               => 0,
-        maxAwaitTimeMS      => 0,
-        maxTimeMS           => 0,
-        noCursorTimeout     => 0,
-        oplogReplay         => 0,
-        skip                => 0,
-        sort                => undef,
-        %$options,
-        filter     => $filter     || {},
-        projection => $projection || {},
-        limit      => -1,
-        %{ $self->_op_args },
-    )->execute->next;
+    return $self->client->send_read_op(
+        MongoDB::Op::_Query->_new(
+            modifiers           => {},
+            allowPartialResults => 0,
+            batchSize           => 0,
+            comment             => '',
+            cursorType          => 'non_tailable',
+            limit               => 0,
+            maxAwaitTimeMS      => 0,
+            maxTimeMS           => 0,
+            noCursorTimeout     => 0,
+            oplogReplay         => 0,
+            skip                => 0,
+            sort                => undef,
+            %$options,
+            filter     => $filter     || {},
+            projection => $projection || {},
+            limit      => -1,
+            %{ $self->_op_args },
+        )
+    )->next;
 }
 
 #pod =method find_id
@@ -803,6 +856,9 @@ sub find_id {
 #pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
 #pod * C<projection> - a hash reference defining fields to return. See "L<limit
@@ -859,6 +915,9 @@ sub find_one_and_delete {
 #pod =for :list
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
 #pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.
 #pod * C<projection> - a hash reference defining fields to return. See "L<limit
@@ -901,6 +960,9 @@ sub find_one_and_replace {
 #pod =for :list
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
 #pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
 #pod * C<projection> - a hash reference defining fields to return. See "L<limit
@@ -952,6 +1014,9 @@ sub find_one_and_update {
 #pod * C<batchSize> – the number of documents to return per batch.
 #pod * C<bypassDocumentValidation> - skips document validation, if enabled.
 #pod   (Note, this will be ignored for servers before version 3.2.)
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<explain> – if true, return a single document with execution information.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
@@ -993,17 +1058,13 @@ sub aggregate {
 
     # read preferences are ignored if the last stage is $out
     my ($last_op) = keys %{ $pipeline->[-1] };
-    my $read_pref = $last_op eq '$out' ? undef : $self->read_preference;
 
     my $op = MongoDB::Op::_Aggregate->_new(
-        db_name    => $self->database->name,
-        coll_name  => $self->name,
-        client     => $self->client,
-        bson_codec => $self->bson_codec,
-        pipeline   => $pipeline,
-        options    => $options,
-        ( $read_pref ? ( read_preference => $read_pref ) : () ),
+        pipeline     => $pipeline,
+        options      => $options,
         read_concern => $self->read_concern,
+        has_out      => $last_op eq '$out',
+        %{ $self->_op_args },
     );
 
     return $self->client->send_read_op($op);
@@ -1019,6 +1080,9 @@ sub aggregate {
 #pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<hint> – L<specify an index to
 #pod   use|http://docs.mongodb.org/manual/reference/command/count/#specify-the-index-to-use>;
 #pod   must be a string, array reference, hash reference or L<Tie::IxHash> object.
@@ -1074,6 +1138,9 @@ sub count {
 #pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
+#pod * C<collation> - a L<document|/Document> defining the collation for this operation.
+#pod   See docs for the format of the collation document here:
+#pod   L<https://docs.mongodb.com/master/reference/collation/>.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
 #pod
@@ -1099,15 +1166,10 @@ sub distinct {
     }
 
     my $op = MongoDB::Op::_Distinct->_new(
-        db_name         => $self->database->name,
-        coll_name       => $self->name,
-        client          => $self->client,
-        bson_codec      => $self->bson_codec,
         fieldname       => $fieldname,
         filter          => $filter,
         options         => $options,
-        read_preference => $self->read_preference,
-        read_concern    => $self->read_concern,
+        %{ $self->_op_args },
     );
 
     return $self->client->send_read_op($op);
@@ -1117,6 +1179,7 @@ sub distinct {
 #pod =method parallel_scan
 #pod
 #pod     @result_objs = $collection->parallel_scan(10);
+#pod     @result_objs = $collection->parallel_scan(10, $options );
 #pod
 #pod Returns one or more L<MongoDB::QueryResult> objects to scan the collection in
 #pod parallel. The argument is the maximum number of L<MongoDB::QueryResult> objects
@@ -1128,22 +1191,27 @@ sub distinct {
 #pod B<Note>: the server may return fewer cursors than requested, depending on the
 #pod underlying storage engine and resource availability.
 #pod
+#pod A hash reference of options may be provided. Valid keys include:
+#pod
+#pod =for :list
+#pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
+#pod   command to run.  (Note, this will be ignored for servers before version 3.4.)
+#pod
 #pod =cut
 
 sub parallel_scan {
-    my ( $self, $num_cursors, $opts ) = @_;
+    my ( $self, $num_cursors, $options ) = @_;
     unless (defined $num_cursors && $num_cursors == int($num_cursors)
         && $num_cursors > 0 && $num_cursors <= 10000
     ) {
         MongoDB::UsageError->throw( "first argument to parallel_scan must be a positive integer between 1 and 10000" )
     }
-    $opts = ref $opts eq 'HASH' ? $opts : { };
-
-    my $db   = $self->database;
+    $options = ref $options eq 'HASH' ? $options : { };
 
     my $op = MongoDB::Op::_ParallelScan->_new(
         %{ $self->_op_args },
         num_cursors     => $num_cursors,
+        options         => $options,
     );
 
     my $result = $self->client->send_read_op( $op );
@@ -1156,10 +1224,10 @@ sub parallel_scan {
     for my $c ( map { $_->{cursor} } @{$response->{cursors}} ) {
         my $batch = $c->{firstBatch};
         my $qr = MongoDB::QueryResult->_new(
-            _client    => $self->client,
-            _address    => $result->address,
-            _ns         => $c->{ns},
-            _bson_codec => $self->bson_codec,
+            _client       => $self->client,
+            _address      => $result->address,
+            _full_name    => $c->{ns},
+            _bson_codec   => $self->bson_codec,
             _batch_size   => scalar @$batch,
             _cursor_at    => 0,
             _limit        => 0,
@@ -1167,7 +1235,7 @@ sub parallel_scan {
             _cursor_start => 0,
             _cursor_flags => {},
             _cursor_num   => scalar @$batch,
-            _docs        => $batch,
+            _docs         => $batch,
         );
         push @cursors, $qr;
     }
@@ -1188,20 +1256,17 @@ sub parallel_scan {
 #pod =cut
 
 sub rename {
-    my ($self, $collectionname) = @_;
+    my ( $self, $new_name ) = @_;
 
-    my $conn = $self->client;
-    my $database = $conn->get_database( 'admin' );
-    my $fullname = $self->full_name;
+    my $op = MongoDB::Op::_RenameCollection->_new(
+        src_ns => $self->full_name,
+        dst_ns => join( ".", $self->database->name, $new_name ),
+        %{ $self->_op_args },
+    );
 
-    my ($db, @collection_bits) = split(/\./, $fullname);
-    my $collection = join('.', @collection_bits);
+    $self->client->send_write_op($op);
 
-    # this does NOT use our private _run_command method as it needs to run
-    # against a totally different database
-    my $obj = $database->run_command([ 'renameCollection' => "$db.$collection", 'to' => "$db.$collectionname" ]);
-
-    return $conn->get_database( $db )->get_collection( $collectionname );
+    return $self->database->get_collection($new_name);
 }
 
 #pod =method drop
@@ -1214,20 +1279,17 @@ sub rename {
 
 sub drop {
     my ($self) = @_;
-    try {
-        $self->_run_command({ drop => $self->name });
-    }
-    catch {
-        die $_ unless /ns not found/;
-    };
+
+    $self->client->send_write_op( MongoDB::Op::_DropCollection->_new( %{ $self->_op_args } ) );
+
     return;
 }
 
 #pod =method ordered_bulk
 #pod
 #pod     $bulk = $coll->ordered_bulk;
-#pod     $bulk->insert( $doc1 );
-#pod     $bulk->insert( $doc2 );
+#pod     $bulk->insert_one( $doc1 );
+#pod     $bulk->insert_one( $doc2 );
 #pod     ...
 #pod     $result = $bulk->execute;
 #pod
@@ -1384,10 +1446,17 @@ sub bulk_write {
             $bulk->insert_one($_) for @$args;
         }
         else {
-            my ($filter, $doc, $opts) = @$args;
+            my ( $filter, $arg2, $arg3 ) = @$args;
+
+            my $is_delete = $method eq 'delete_one' || $method eq 'delete_many';
+            my $update_doc = $is_delete ? undef : $arg2;
+            my $opts       = $is_delete ? $arg2 : $arg3;
 
             my $view = $bulk->find($filter);
 
+            # set collation
+            $view = $view->collation( $opts->{collation} ) if $opts && $opts->{collation};
+
             # handle deletes
             if ( $method eq 'delete_one' ) {
                 $view->delete_one;
@@ -1403,13 +1472,13 @@ sub bulk_write {
 
             # handle updates
             if ( $method eq 'replace_one' ) {
-                $view->replace_one($doc);
+                $view->replace_one($update_doc);
             }
             elsif ( $method eq 'update_one' ) {
-                $view->update_one($doc);
+                $view->update_one($update_doc);
             }
             elsif ( $method eq 'update_many' ) {
-                $view->update_many($doc);
+                $view->update_many($update_doc);
             }
             else {
                 MongoDB::UsageError->throw("unknown bulk operation '$method'");
@@ -1478,28 +1547,6 @@ sub _find_one_and_update_or_replace {
     return $self->client->send_write_op($op);
 }
 
-# we have a private _run_command rather than using the 'database' attribute
-# so that we're using our BSON codec and not the source database one
-sub _run_command {
-    my ( $self, $command, $read_pref ) = @_;
-
-    if ( $read_pref && ref($read_pref) eq 'HASH' ) {
-        $read_pref = MongoDB::ReadPreference->new($read_pref);
-    }
-
-    my $op = MongoDB::Op::_Command->_new(
-        db_name     => $self->database->name,
-        query       => $command,
-        query_flags => {},
-        bson_codec  => $self->bson_codec,
-        ( $read_pref ? ( read_preference => $read_pref ) : () ),
-    );
-
-    my $obj = $self->client->send_read_op($op);
-
-    return $obj->output;
-}
-
 #--------------------------------------------------------------------------#
 # utility function
 #--------------------------------------------------------------------------#
@@ -1534,6 +1581,8 @@ sub insert {
 
     my ( $self, $document, $opts ) = @_;
 
+    $self->_warn_deprecated( 'insert' => ['insert_one'] );
+
     my $op = MongoDB::Op::_InsertOne->_new(
         document => $document,
         %{ $self->_op_args },
@@ -1551,18 +1600,20 @@ sub batch_insert {
 
     my ( $self, $documents, $opts ) = @_;
 
+    $self->_warn_deprecated( 'batch_insert' => ['insert_many'] );
+
     my $op = MongoDB::Op::_BatchInsert->_new(
-        db_name       => $self->database->name,
-        coll_name     => $self->name,
-        bson_codec    => $self->bson_codec,
         documents     => $documents,
-        write_concern => $self->_dynamic_write_concern($opts),
         check_keys    => 0,
         ordered       => 1,
+        %{ $_[0]->_op_args },
+        write_concern => $self->_dynamic_write_concern($opts),
     );
 
     my $result = $self->client->send_write_op($op);
 
+    return if $result->isa("MongoDB::UnacknowledgedResult");
+
     my @ids;
     my $inserted_ids = $result->inserted_ids;
     for my $k ( sort { $a <=> $b } keys %$inserted_ids ) {
@@ -1576,9 +1627,15 @@ sub remove {
     my ($self, $query, $opts) = @_;
     $opts ||= {};
 
+    $self->_warn_deprecated( 'remove' => [qw/delete_many delete_one/] );
+
+    MongoDB::UsageError->throw(
+        "deprecated method 'remove' does not support a collation, use one of its replacement methods instead"
+    ) if exists $opts->{collation};
+
     my $op = MongoDB::Op::_Delete->_new(
-        filter        => $query || {},
-        just_one      => !! $opts->{just_one},
+        filter => $query || {},
+        just_one => !!$opts->{just_one},
         %{ $self->_op_args },
         write_concern => $self->_dynamic_write_concern($opts),
     );
@@ -1596,6 +1653,13 @@ my $legacy_update_args;
 sub update {
     my ( $self, $query, $object, $opts ) = @_;
     $opts ||= {};
+    $object ||= {};
+
+    $self->_warn_deprecated( 'update' => [qw/update_one update_many replace_one/] );
+
+    MongoDB::UsageError->throw(
+        "deprecated method 'update' does not support a collation, use one of its replacement methods instead"
+    ) if exists $opts->{collation};
 
     if ( exists $opts->{multiple} ) {
         if ( exists( $opts->{multi} ) && !!$opts->{multi} ne !!$opts->{multiple} ) {
@@ -1615,8 +1679,8 @@ sub update {
     );
     $fk = defined($fk) ? substr($fk,0,1) : '';
 
-    my $op_char = eval { $self->bson_codec->op_char } || '';
-    my $is_replace = $fk ne '$' && $fk ne $op_char;
+    my $op_char = eval { $self->bson_codec->op_char } || '$';
+    my $is_replace = $fk ne $op_char;
 
     my $op = MongoDB::Op::_Update->_new(
         filter => $query  || {},
@@ -1649,6 +1713,8 @@ sub save {
 
     my ($self, $doc, $options) = @_;
 
+    $self->_warn_deprecated( 'save', "Use 'replace_one' with upsert instead." );
+
     my $type = ref($doc);
     my $id = (
           $type eq 'HASH' ? $doc->{_id}
@@ -1675,6 +1741,13 @@ sub find_and_modify {
     my ( $self, $opts ) = @_;
     $opts ||= {};
 
+    $self->_warn_deprecated( 'find_and_modify' =>
+          [qw/find_one_and_update find_one_and_replace find_one_and_delete/] );
+
+    MongoDB::UsageError->throw(
+        "deprecated method 'find_and_modify' does not support a collation, use one of its replacement methods instead"
+    ) if exists $opts->{collation};
+
     MongoDB::UsageError->throw("find_and_modify requires a 'query' option")
         unless $opts->{query};
 
@@ -1694,14 +1767,24 @@ sub get_collection {
     my $self = shift @_;
     my $coll = shift @_;
 
+    $self->_warn_deprecated( 'get_collection',
+        "Use \$coll->database->coll(join('.', \$coll->name, 'subname')) instead." );
+
     return $self->database->get_collection($self->name.'.'.$coll);
 }
 
 sub ensure_index {
     my ( $self, $keys, $opts ) = @_;
+
+    $self->_warn_deprecated( 'ensure_index' => "Use 'indexes' to work with a MongoDB::IndexView instead." );
+
     MongoDB::UsageError->throw("ensure_index options must be a hash reference")
       if $opts && !ref($opts) eq 'HASH';
 
+    MongoDB::UsageError->throw(
+        "deprecated method 'ensure_index' does not support a collation, use 'indexes' instead"
+    ) if exists $opts->{collation};
+
     $keys = Tie::IxHash->new(@$keys) if ref $keys eq 'ARRAY';
     $opts = $self->_clean_index_options( $opts, $keys );
 
@@ -1714,6 +1797,7 @@ sub ensure_index {
     my $op = MongoDB::Op::_CreateIndexes->_new(
         db_name       => $self->database->name,
         coll_name     => $self->name,
+        full_name     => $self->full_name,
         bson_codec    => $self->bson_codec,
         indexes       => [ { key => $keys, %$opts } ],
         write_concern => $wc,
@@ -1785,25 +1869,30 @@ sub __to_index_string {
 sub get_indexes {
     my ($self) = @_;
 
+    $self->_warn_deprecated( 'get_indexes'  => "Use 'indexes' to work with a MongoDB::IndexView instead." );
+
     my $op = MongoDB::Op::_ListIndexes->_new(
-        db_name    => $self->database->name,
-        coll_name  => $self->name,
-        client     => $self->client,
-        bson_codec => $self->bson_codec,
+        %{ $_[0]->_op_args },
     );
 
-    my $res = $self->client->send_read_op($op);
+    my $res = $self->client->send_primary_op($op);
 
     return $res->all;
 }
 
 sub drop_indexes {
     my ($self) = @_;
+
+    $self->_warn_deprecated( 'drop_indexes'  => "Use 'indexes' to work with a MongoDB::IndexView instead." );
+
     return $self->drop_index('*');
 }
 
 sub drop_index {
     my ($self, $index_name) = @_;
+
+    $self->_warn_deprecated( 'drop_index'  => "Use 'indexes' to work with a MongoDB::IndexView instead." );
+
     return $self->_run_command([
         dropIndexes => $self->name,
         index => $index_name,
@@ -1812,10 +1901,29 @@ sub drop_index {
 
 sub validate {
     my ($self, $scan_data) = @_;
+
+    $self->_warn_deprecated( 'validate'  => "Use 'validate' manually via MongoDB::Database::run_command" );
+
     $scan_data = 0 unless defined $scan_data;
     my $obj = $self->_run_command({ validate => $self->name });
 }
 
+# we have a private _run_command rather than using the 'database' attribute
+# so that we're using our BSON codec and not the source database one
+sub _run_command {
+    my ( $self, $command ) = @_;
+
+    my $op = MongoDB::Op::_Command->_new(
+        db_name     => $self->database->name,
+        query       => $command,
+        query_flags => {},
+        bson_codec  => $self->bson_codec,
+    );
+
+    my $obj = $self->client->send_read_op($op);
+
+    return $obj->output;
+}
 
 1;
 
@@ -1829,7 +1937,7 @@ MongoDB::Collection - A MongoDB Collection
 
 =head1 VERSION
 
-version v1.2.2
+version v1.8.1
 
 =head1 SYNOPSIS
 
@@ -1871,7 +1979,7 @@ call C<get_collection> on a L<MongoDB::Database> object.
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -1881,7 +1989,7 @@ are recommended:
     use Safe::Isa; # provides $_isa
 
     try {
-        $coll->insert( $doc )
+        $coll->insert_one( $doc )
     }
     catch {
         if ( $_->$_isa("MongoDB::DuplicateKeyError" ) {
@@ -2080,19 +2188,43 @@ calls.
 
     $res = $coll->delete_one( $filter );
     $res = $coll->delete_one( { _id => $id } );
+    $res = $coll->delete_one( $filter, { collation => { locale => "en_US" } } );
 
 Deletes a single document that matches a L<filter expression|/Filter expression> and returns a
 L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult> object.
 
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=back
+
 =head2 delete_many
 
     $res = $coll->delete_many( $filter );
     $res = $coll->delete_many( { name => "Larry" } );
+    $res = $coll->delete_many( $filter, { collation => { locale => "en_US" } } );
 
 Deletes all documents that match a L<filter expression|/Filter expression>
 and returns a L<MongoDB::DeleteResult> or L<MongoDB::UnacknowledgedResult>
 object.
 
+Valid options include:
+
+=over 4
+
+=item *
+
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=back
+
 =head2 replace_one
 
     $res = $coll->replace_one( $filter, $replacement );
@@ -2117,6 +2249,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled; this is ign
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<upsert> – defaults to false; if true, a new document will be added if one is not found
 
 =back
@@ -2145,6 +2281,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled; this is ign
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<upsert> – defaults to false; if true, a new document will be added if one is not found by taking the filter expression and applying the update document operations to it prior to insertion.
 
 =back
@@ -2173,6 +2313,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled; this is ign
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<upsert> – defaults to false; if true, a new document will be added if one is not found by taking the filter expression and applying the update document operations to it prior to insertion.
 
 =back
@@ -2204,6 +2348,10 @@ C<batchSize> – the number of documents to return per batch.
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<comment> – attaches a comment to the query. If C<$comment> also exists in the C<modifiers> document, the comment field overwrites C<$comment>.
 
 =item *
@@ -2224,7 +2372,7 @@ C<maxTimeMS> – the maximum amount of time to allow the query to run. If C<$max
 
 =item *
 
-C<modifiers> – a hash reference of L<query modifiers|http://docs.mongodb.org/manual/reference/operator/query-modifier/> modifying the output or behavior of a query.
+C<modifiers> – a hash reference of dollar-prefixed L<query modifiers|http://docs.mongodb.org/manual/reference/operator/query-modifier/> modifying the output or behavior of a query.
 
 =item *
 
@@ -2286,6 +2434,10 @@ include:
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
 =item *
@@ -2325,6 +2477,10 @@ A hash reference of options may be provided. Valid keys include:
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
 =item *
@@ -2359,6 +2515,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled; this is ign
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.
 
 =item *
@@ -2400,6 +2560,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled; this is ign
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
 =item *
@@ -2455,6 +2619,10 @@ C<bypassDocumentValidation> - skips document validation, if enabled. (Note, this
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<explain> – if true, return a single document with execution information.
 
 =item *
@@ -2491,6 +2659,10 @@ A hash reference of options may be provided. Valid keys include:
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<hint> – L<specify an index to use|http://docs.mongodb.org/manual/reference/command/count/#specify-the-index-to-use>; must be a string, array reference, hash reference or L<Tie::IxHash> object.
 
 =item *
@@ -2531,6 +2703,10 @@ A hash reference of options may be provided. Valid keys include:
 
 =item *
 
+C<collation> - a L<document|/Document> defining the collation for this operation. See docs for the format of the collation document here: L<https://docs.mongodb.com/master/reference/collation/>.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
 =back
@@ -2542,6 +2718,7 @@ details.
 =head2 parallel_scan
 
     @result_objs = $collection->parallel_scan(10);
+    @result_objs = $collection->parallel_scan(10, $options );
 
 Returns one or more L<MongoDB::QueryResult> objects to scan the collection in
 parallel. The argument is the maximum number of L<MongoDB::QueryResult> objects
@@ -2553,6 +2730,16 @@ appear only once in one of the cursors' result sets.
 B<Note>: the server may return fewer cursors than requested, depending on the
 underlying storage engine and resource availability.
 
+A hash reference of options may be provided. Valid keys include:
+
+=over 4
+
+=item *
+
+C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 3.4.)
+
+=back
+
 =head2 rename
 
     $newcollection = $collection->rename("mynewcollection");
@@ -2572,8 +2759,8 @@ Deletes a collection as well as all of its indexes.
 =head2 ordered_bulk
 
     $bulk = $coll->ordered_bulk;
-    $bulk->insert( $doc1 );
-    $bulk->insert( $doc2 );
+    $bulk->insert_one( $doc1 );
+    $bulk->insert_one( $doc2 );
     ...
     $result = $bulk->execute;
 
@@ -2728,7 +2915,7 @@ The C<get_collection> method is deprecated; it implied a 'subcollection'
 relationship that is purely notional.
 
 The C<ensure_index>, C<drop_indexes>, C<drop_index>, and C<get_index>
-methods are deprecated. The new L<MongoDB::IndexView> class is accessable
+methods are deprecated. The new L<MongoDB::IndexView> class is accessible
 through the C<indexes> method, and offer greater consistency in behavior
 across drivers.
 
@@ -2748,6 +2935,10 @@ David Golden <david@mongodb.com>
 
 =item *
 
+Rassi <rassi@mongodb.com>
+
+=item *
+
 Mike Friedman <friedo@friedo.com>
 
 =item *
@@ -2762,7 +2953,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc..
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
