@@ -12,15 +12,15 @@
 #  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
-#
 
+use strict;
+use warnings;
 package MongoDB::GridFS;
 
-
-# ABSTRACT: A file storage utility
+# ABSTRACT: A file storage abstraction (DEPRECATED)
 
 use version;
-our $VERSION = 'v1.2.2';
+our $VERSION = 'v1.8.1';
 
 use MongoDB::GridFS::File;
 use Digest::MD5;
@@ -178,8 +178,8 @@ sub _ensure_indexes {
     my ($self) = @_;
 
     # ensure the necessary index is present (this may be first usage)
-    $self->files->ensure_index(Tie::IxHash->new(filename => 1), {"safe" => 1});
-    $self->chunks->ensure_index(Tie::IxHash->new(files_id => 1, n => 1), {"safe" => 1, "unique" => 1});
+    $self->files->indexes->create_one(Tie::IxHash->new(filename => 1));
+    $self->chunks->indexes->create_one(Tie::IxHash->new(files_id => 1, n => 1), {"unique" => 1});
 }
 
 #pod =method get
@@ -247,7 +247,7 @@ sub find_one {
     $criteria ||= {};
 
     my $file = $self->files->find_one($criteria, $fields);
-    return undef unless $file;
+    return undef unless $file; ## no critic
     return MongoDB::GridFS::File->new({_grid => $self,info => $file});
 }
 
@@ -472,11 +472,11 @@ __END__
 
 =head1 NAME
 
-MongoDB::GridFS - A file storage utility
+MongoDB::GridFS - A file storage abstraction (DEPRECATED)
 
 =head1 VERSION
 
-version v1.2.2
+version v1.8.1
 
 =head1 SYNOPSIS
 
@@ -504,7 +504,7 @@ using one over the other is a matter of preference.
 
 =head2 Error handling
 
-Unless otherwise explictly documented, all methods throw exceptions if
+Unless otherwise explicitly documented, all methods throw exceptions if
 an error occurs.  The error types are documented in L<MongoDB::Error>.
 
 To catch and handle errors, the L<Try::Tiny> and L<Safe::Isa> modules
@@ -673,6 +673,14 @@ Removes all files' metadata and contents.
 Returns a list of the files in the database as L<MongoDB::GridFS::File>
 objects.
 
+=head1 DEPRECATION
+
+B<Note>: This class has been deprecated in favor of
+L<MongoDB::GridFSBucket>, which implements the new, driver-standard GridFS
+API.  It is also faster and more flexible than this class.  This class will
+be removed in a future release and you are encouraged to migrate your
+applications to L<MongoDB::GridFSBucket>.
+
 =head1 SEE ALSO
 
 Core documentation on GridFS: L<http://dochub.mongodb.org/core/gridfs>.
@@ -687,6 +695,10 @@ David Golden <david@mongodb.com>
 
 =item *
 
+Rassi <rassi@mongodb.com>
+
+=item *
+
 Mike Friedman <friedo@friedo.com>
 
 =item *
@@ -701,7 +713,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc..
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
