@@ -14,12 +14,14 @@
 #  limitations under the License.
 #
 
+use strict;
+use warnings;
 package MongoDB::MongoClient;
 
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v1.2.2';
+our $VERSION = 'v1.8.1';
 
 use Moo;
 use MongoDB;
@@ -51,6 +53,8 @@ use MongoDB::_Types qw(
     ArrayOfHashRef
     AuthMechanism
     BSONCodec
+    HeartbeatFreq
+    MaxStalenessNum
     NonNegNum
     ReadPrefMode
     ReadPreference
@@ -104,6 +108,41 @@ has host => (
     default => 'mongodb://localhost:27017',
 );
 
+#pod =attr app_name
+#pod
+#pod This attribute specifies an application name that should be associated with
+#pod this client.  The application name will be communicated to the server as
+#pod part of the initial connection handshake, and will appear in
+#pod connection-level and operation-level diagnostics on the server generated on
+#pod behalf of this client.  This may be set in a connection string with the
+#pod C<appName> option.
+#pod
+#pod The default is the empty string, which indicates a lack of an application
+#pod name.
+#pod
+#pod The application name must not exceed 128 bytes.
+#pod
+#pod =cut
+
+has app_name => (
+    is  => 'lazy',
+    isa => Str,
+    builder => '_build_app_name',
+);
+
+sub _build_app_name {
+    my ($self) = @_;
+    my $app_name = $self->__uri_or_else(
+        u => 'appname',
+        e => 'app_name',
+        d => '',
+    );
+    unless ( length($app_name) <= 128 ) {
+        MongoDB::UsageError->throw("app name must be at most 128 bytes");
+    }
+    return $app_name;
+}
+
 #pod =attr auth_mechanism
 #pod
 #pod This attribute determines how the client authenticates with the server.
@@ -288,7 +327,7 @@ sub _build_db_name {
 
 has heartbeat_frequency_ms => (
     is      => 'lazy',
-    isa     => NonNegNum,
+    isa     => HeartbeatFreq,
     builder => '_build_heartbeat_frequency_ms',
 );
 
@@ -362,6 +401,42 @@ sub _build_local_threshold_ms {
     );
 }
 
+#pod =attr max_staleness_seconds
+#pod
+#pod The C<max_staleness_seconds> parameter represents the maximum replication lag in
+#pod seconds (wall clock time) that a secondary can suffer and still be
+#pod eligible for reads. The default is -1, which disables staleness checks.
+#pod Otherwise, it must be a positive integer.
+#pod
+#pod B<Note>: this will only be used for server versions 3.4 or greater, as that
+#pod was when support for staleness tracking was added.
+#pod
+#pod If the read preference mode is 'primary', then C<max_staleness_seconds> must not
+#pod be supplied.
+#pod
+#pod The C<max_staleness_seconds> must be at least the C<heartbeat_frequency_ms>
+#pod plus 10 seconds (which is how often the server makes idle writes to the
+#pod oplog).
+#pod
+#pod This may be set in a connection string with the C<maxStalenessSeconds> option.
+#pod
+#pod =cut
+
+has max_staleness_seconds => (
+    is      => 'lazy',
+    isa     => MaxStalenessNum,
+    builder => '_build_max_staleness_seconds',
+);
+
+sub _build_max_staleness_seconds {
+    my ($self) = @_;
+    return $self->__uri_or_else(
+        u => 'maxstalenessseconds',
+        e => 'max_staleness_seconds',
+        d => -1,
+    );
+}
+
 #pod =attr max_time_ms
 #pod
 #pod Specifies the maximum amount of time in (non-negative) milliseconds that the
@@ -971,8 +1046,9 @@ has _read_preference => (
 sub _build__read_preference {
     my ($self) = @_;
     return MongoDB::ReadPreference->new(
-        ( $self->read_pref_mode     ? ( mode     => $self->read_pref_mode )     : () ),
-        ( $self->read_pref_tag_sets ? ( tag_sets => $self->read_pref_tag_sets ) : () ),
+        ( defined $self->read_pref_mode     ? ( mode     => $self->read_pref_mode )     : () ),
+        ( defined $self->read_pref_tag_sets ? ( tag_sets => $self->read_pref_tag_sets ) : () ),
+        ( defined $self->max_staleness_seconds ? ( max_staleness_seconds => $self->max_staleness_seconds ) : () ),
     );
 }
 
@@ -1067,13 +1143,14 @@ sub _build__topology {
 
     my $type =
         length( $self->replica_set_name ) ? 'ReplicaSetNoPrimary'
-      : @{ $self->_uri->hostpairs } > 1   ? 'Sharded'
+      : @{ $self->_uri->hostids } > 1   ? 'Sharded'
       :                                     'Single';
 
     MongoDB::_Topology->new(
         uri                         => $self->_uri,
         bson_codec                  => $self->bson_codec,
         type                        => $type,
+        app_name                    => $self->app_name,
         replica_set_name            => $self->replica_set_name,
         server_selection_timeout_sec => $self->server_selection_timeout_ms / 1000,
         server_selection_try_once    => $self->server_selection_try_once,
@@ -1138,6 +1215,7 @@ sub _build__uri {
 # these attributes are lazy, built from either _uri->options or from
 # _config_options captured in BUILDARGS
 my @deferred_options = qw(
+  app_name
   auth_mechanism
   auth_mechanism_properties
   connect_timeout_ms
@@ -1145,6 +1223,7 @@ my @deferred_options = qw(
   heartbeat_frequency_ms
   j
   local_threshold_ms
+  max_staleness_seconds
   max_time_ms
   read_pref_mode
   read_pref_tag_sets
@@ -1179,7 +1258,7 @@ sub BUILD {
 
     my $uri = $self->_uri;
 
-    my @addresses = @{ $uri->hostpairs };
+    my @addresses = @{ $uri->hostids };
 
     # resolve and validate all deferred attributes
     $self->$_ for @deferred_options;
@@ -1208,6 +1287,9 @@ sub BUILD {
         );
     }
 
+    # Instantiate topology
+    $self->_topology;
+
     return;
 }
 
@@ -1361,15 +1443,15 @@ sub send_admin_command {
 sub send_direct_op {
     my ( $self, $op, $address ) = @_;
     my ( $link, $result );
-    ( $link = $self->_topology->get_specific_link($address) ), (
+    ( $link = $self->{_topology}->get_specific_link($address) ), (
         eval { ($result) = $op->execute($link); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
             }
             elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
-                $self->_topology->mark_stale;
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_stale;
             }
             # regardless of cleanup, rethrow the error
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
@@ -1382,15 +1464,15 @@ sub send_direct_op {
 sub send_write_op {
     my ( $self, $op ) = @_;
     my ( $link, $result );
-    ( $link = $self->_topology->get_writable_link ), (
-        eval { ($result) = $op->execute($link); 1 } or do {
+    ( $link = $self->{_topology}->get_writable_link ), (
+        eval { ($result) = $op->execute($link, $self->{_topology}->type); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
             }
             elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
-                $self->_topology->mark_stale;
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_stale;
             }
             # regardless of cleanup, rethrow the error
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
@@ -1399,20 +1481,28 @@ sub send_write_op {
       return $result;
 }
 
+# Sometimes, seeing an op dispatched as "send_write_op" is confusing when
+# really, we're just insisting that it be sent only to a primary or
+# directly connected server.
+BEGIN {
+    no warnings 'once';
+    *send_primary_op = \&send_write_op;
+}
+
 # op dispatcher written in highly optimized style
 sub send_read_op {
     my ( $self, $op ) = @_;
     my ( $link, $type, $result );
-    ( $link = $self->_topology->get_readable_link( $op->read_preference ) ),
-      ( $type = $self->_topology->type ), (
+    ( $link = $self->{_topology}->get_readable_link( $op->read_preference ) ),
+      ( $type = $self->{_topology}->type ), (
         eval { ($result) = $op->execute( $link, $type ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             if ( $err->$_isa("MongoDB::ConnectionError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
             }
             elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
-                $self->_topology->mark_server_unknown( $link->server, $err );
-                $self->_topology->mark_stale;
+                $self->{_topology}->mark_server_unknown( $link->server, $err );
+                $self->{_topology}->mark_stale;
             }
             # regardless of cleanup, rethrow the error
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
@@ -1564,7 +1654,7 @@ sub fsync_unlock {
         bson_codec => $self->bson_codec,
     );
 
-    return $self->send_read_op($op);
+    return $self->send_primary_op($op);
 }
 
 
@@ -1582,7 +1672,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v1.2.2
+version v1.8.1
 
 =head1 SYNOPSIS
 
@@ -1657,6 +1747,20 @@ For IPv6 support, you must have a recent version of L<IO::Socket::IP>
 installed.  This module ships with the Perl core since v5.20.0 and is
 available on CPAN for older Perls.
 
+=head2 app_name
+
+This attribute specifies an application name that should be associated with
+this client.  The application name will be communicated to the server as
+part of the initial connection handshake, and will appear in
+connection-level and operation-level diagnostics on the server generated on
+behalf of this client.  This may be set in a connection string with the
+C<appName> option.
+
+The default is the empty string, which indicates a lack of an application
+name.
+
+The application name must not exceed 128 bytes.
+
 =head2 auth_mechanism
 
 This attribute determines how the client authenticates with the server.
@@ -1796,6 +1900,25 @@ See L</SERVER SELECTION> for more details.
 
 This may be set in a connection string with the C<localThresholdMS> option.
 
+=head2 max_staleness_seconds
+
+The C<max_staleness_seconds> parameter represents the maximum replication lag in
+seconds (wall clock time) that a secondary can suffer and still be
+eligible for reads. The default is -1, which disables staleness checks.
+Otherwise, it must be a positive integer.
+
+B<Note>: this will only be used for server versions 3.4 or greater, as that
+was when support for staleness tracking was added.
+
+If the read preference mode is 'primary', then C<max_staleness_seconds> must not
+be supplied.
+
+The C<max_staleness_seconds> must be at least the C<heartbeat_frequency_ms>
+plus 10 seconds (which is how often the server makes idle writes to the
+oplog).
+
+This may be set in a connection string with the C<maxStalenessSeconds> option.
+
 =head2 max_time_ms
 
 Specifies the maximum amount of time in (non-negative) milliseconds that the
@@ -2344,6 +2467,7 @@ The currently supported connection string options are:
 
 =over 4
 
+*appName
 *authMechanism
 *authMechanism.SERVICE_NAME
 *connectTimeoutMS
@@ -2505,19 +2629,9 @@ connection string.
 
 =head2 MONGODB-X509 (for SSL client certificate)
 
-X509 authentication requires SSL support (L<IO::Socket::SSL>) and requires
-that a client certificate be configured and that the username attribute be
-set to the "Subject" field, formatted according to RFC 2253.  To find the
-correct username, run the C<openssl> program as follows:
-
-  $ openssl x509 -in certs/client.pem -inform PEM -subject -nameopt RFC2253
-  subject= CN=XXXXXXXXXXX,OU=XXXXXXXX,O=XXXXXXX,ST=XXXXXXXXXX,C=XX
-
-In this case the C<username> attribute would be
-C<CN=XXXXXXXXXXX,OU=XXXXXXXX,O=XXXXXXX,ST=XXXXXXXXXX,C=XX>.
-
-Configure your client with the correct username and ssl parameters, and
-specify the "MONGODB-X509" authentication mechanism.
+X509 authentication requires SSL support (L<IO::Socket::SSL>), requires
+that a client certificate be configured in the ssl parameters, and requires
+specifying the "MONGODB-X509" authentication mechanism.
 
     my $mc = MongoDB::MongoClient->new(
         host => "mongodb://sslmongo.example.com/",
@@ -2526,7 +2640,6 @@ specify the "MONGODB-X509" authentication mechanism.
             SSL_cert_file => "certs/client.pem",
         },
         auth_mechanism => "MONGODB-X509",
-        username       => "CN=XXXXXXXXXXX,OU=XXXXXXXX,O=XXXXXXX,ST=XXXXXXXXXX,C=XX"
     );
 
 =head2 PLAIN (for LDAP)
@@ -2618,6 +2731,10 @@ David Golden <david@mongodb.com>
 
 =item *
 
+Rassi <rassi@mongodb.com>
+
+=item *
+
 Mike Friedman <friedo@friedo.com>
 
 =item *
@@ -2632,7 +2749,7 @@ Florian Ragwitz <rafl@debian.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by MongoDB, Inc..
+This software is Copyright (c) 2018 by MongoDB, Inc.
 
 This is free software, licensed under:
 
