@@ -14,46 +14,24 @@
 #  limitations under the License.
 #
 
+use strict;
+use warnings;
 package MongoDB::Op::_InsertOne;
 
 # Encapsulate a single-document insert operation; returns a
 # MongoDB::InsertOneResult
 
 use version;
-our $VERSION = 'v1.4.5';
+our $VERSION = 'v1.8.1';
 
 use Moo;
 
-use MongoDB::BSON;
 use MongoDB::Error;
 use MongoDB::InsertOneResult;
-use MongoDB::OID;
-use MongoDB::_Constants;
 use MongoDB::_Protocol;
-use Types::Standard qw(
-    Str
-);
-use boolean;
+
 use namespace::clean;
 
-has db_name => (
-    is       => 'ro',
-    required => 1,
-    isa      => Str,
-);
-
-has coll_name => (
-    is       => 'ro',
-    required => 1,
-    isa      => Str,
-);
-
-has full_name => (
-    is       => 'ro',
-    required => 1,
-    isa      => Str,
-);
-
 has document => (
     is       => 'ro',
     required => 1,
@@ -68,7 +46,8 @@ has _doc_id => (
 
 with $_ for qw(
   MongoDB::Role::_PrivateConstructor
-  MongoDB::Role::_WriteCommand
+  MongoDB::Role::_CollectionOp
+  MongoDB::Role::_SingleBatchDocWrite
   MongoDB::Role::_InsertPreEncoder
   MongoDB::Role::_BypassValidation
 );
